package org.siqisource.agave.test;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.dbunit.Assertion;
import org.dbunit.DatabaseUnitException;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.dataset.DefaultDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.excel.XlsDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.test.context.TestContext;
import org.springframework.test.context.support.AbstractTestExecutionListener;

public class XlsDataSetTestExecutionListener extends AbstractTestExecutionListener {

	protected final Logger log = LoggerFactory.getLogger(getClass());

	public void beforeTestMethod(TestContext testContext) throws Exception {

		Class<?> testClass = testContext.getTestClass();
		Method testMethod = testContext.getTestMethod();
		DataSet annotation = (DataSet) AnnotationUtils.findAnnotation(testMethod, DataSet.class);
		if (annotation == null) {
			return;
		}

		final InputStream ins = getFileInputStream(testClass, annotation);
		DatabaseConnection databaseConnection = getDatabaseConnection(testContext, annotation);

		IDataSet dataSet = new XlsDataSet(ins);
		String[] tables = annotation.tables();
		for (int i = 0; i < tables.length; i++) {
			String tablePairs = tables[i];
			ITable preTable = dataSet.getTable(tablePairs.split(",")[0]);
			DefaultDataSet dds = new DefaultDataSet(preTable);
			DatabaseOperation.CLEAN_INSERT.execute(databaseConnection, dds);

		}

		ins.close();
		databaseConnection.close();
	}

	public void afterTestMethod(TestContext testContext) throws Exception {
		Class<?> testClass = testContext.getTestClass();
		Method testMethod = testContext.getTestMethod();
		DataSet annotation = (DataSet) AnnotationUtils.findAnnotation(testMethod, DataSet.class);
		if (annotation == null) {
			return;
		}
		final InputStream ins = getFileInputStream(testClass, annotation);
		DatabaseConnection databaseConnection = getDatabaseConnection(testContext, annotation);

		IDataSet dataSet = new XlsDataSet(ins);
		String[] tables = annotation.tables();

		IDataSet databaseDataSet = databaseConnection.createDataSet();
		for (int i = 0; i < tables.length; i++) {
			String tablePairs = tables[i];
			if (tablePairs.indexOf(',') == -1) {
				continue;
			}
			String[] tableNames = tablePairs.split(",");

			ITable actualTable = databaseDataSet.getTable(tableNames[0].trim());
			ITable expectedTable = dataSet.getTable(tableNames[1].trim());

			Assertion.assertEquals(expectedTable, actualTable);
		}
		ins.close();
		databaseConnection.close();
	}

	public static String getPackageName(Class<?> clazz) {
		String className = clazz.getName();
		int lastDotIndex = className.lastIndexOf(".");
		return (lastDotIndex != -1 ? className.substring(0, lastDotIndex + 1) : "");
	}

	public static InputStream getFileInputStream(Class<?> testClass, DataSet annotation) throws IOException {
		String filePath = "classpath:/" + getPackageName(testClass).replace('.', '/');
		String file = annotation.file();
		if ("".equals(file)) {
			file = testClass.getSimpleName();
		}
		String fileLocation = filePath + file;
		return new DefaultResourceLoader().getResource(fileLocation).getInputStream();

	}

	public static DatabaseConnection getDatabaseConnection(TestContext testContext, DataSet annotation)
			throws SQLException, DatabaseUnitException {
		DataSource datasource = (DataSource) testContext.getApplicationContext().getBean(annotation.dataSource());
		Connection connection = datasource.getConnection();
		return new DatabaseConnection(connection);
	}

}
